@@ -1,21 +1,33 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
+from __future__ import absolute_import,print_function
 
 import os
+import sys
 import code
 import warnings
 import string
+import inspect
 
 import argparse
 
 from flask import _request_ctx_stack
 
 from .cli import prompt, prompt_pass, prompt_bool, prompt_choices
+from ._compat import izip, text_type
 
 
 class InvalidCommand(Exception):
-    pass
+    """\
+        This is a generic error for "bad" commands.
+        It is not used in Flask-Script itself, but you should throw
+        this error (or one derived from it) in your command handlers,
+        and your main code should display this error's message without
+        a stack trace.
 
+        This way, we maintain interoperability if some other plug-in code
+        supplies Flask-Script hooks.
+        """
+    pass
 
 class Group(object):
     """
@@ -90,9 +102,57 @@ class Option(object):
 class Command(object):
     """
     Base class for creating commands.
+
+    :param func:  Initialize this command by introspecting the function.
     """
 
-    option_list = []
+    option_list = ()
+    help_args = None
+
+    def __init__(self, func=None):
+        if func is None:
+            if not self.option_list:
+                self.option_list = []
+            return
+
+        args, varargs, keywords, defaults = inspect.getargspec(func)
+        if inspect.ismethod(func):
+            args = args[1:]
+            
+        options = []
+        
+        # first arg is always "app" : ignore
+        
+        defaults = defaults or []
+        kwargs = dict(izip(*[reversed(l) for l in (args, defaults)]))
+        
+        for arg in args:
+        
+            if arg in kwargs:
+            
+                default = kwargs[arg]
+                
+                if isinstance(default, bool):
+                    options.append(Option('-%s' % arg[0],
+                                          '--%s' % arg,
+                                          action="store_true",
+                                          dest=arg,
+                                          required=False,
+                                          default=default))
+                else:                     
+                    options.append(Option('-%s' % arg[0],
+                                          '--%s' % arg,
+                                          dest=arg,
+                                          type=text_type,
+                                          required=False,
+                                          default=default))
+                                          
+            else:
+                options.append(Option(arg, type=text_type))
+                
+        self.run = func
+        self.__doc__ = func.__doc__
+        self.option_list = options
 
     @property
     def description(self):
@@ -113,8 +173,17 @@ class Command(object):
         return self.option_list
 
     def create_parser(self, *args, **kwargs):
+        func_stack = kwargs.pop('func_stack',())
+        parent = kwargs.pop('parent',None)
+        parser = argparse.ArgumentParser(*args, add_help=False, **kwargs)
+        help_args = self.help_args
+        while help_args is None and parent is not None:
+            help_args = parent.help_args
+            parent = getattr(parent,'parent',None)
 
-        parser = argparse.ArgumentParser(*args, **kwargs)
+        if help_args:
+            from flask_script import add_help
+            add_help(parser,help_args)
 
         for option in self.get_options():
             if isinstance(option, Group):
@@ -132,14 +201,16 @@ class Command(object):
             else:
                 parser.add_argument(*option.args, **option.kwargs)
 
-        parser.set_defaults(func_handle=self.handle)
+        parser.set_defaults(func_stack=func_stack+(self,))
 
+        self.parser = parser
+        self.parent = parent
         return parser
 
-    def handle(self, app, *args, **kwargs):
+    def __call__(self, app=None, *args, **kwargs):
         """
-        Handles the command with given app. Default behaviour is to call within
-        a test request context.
+        Handles the command with the given app.
+        Default behaviour is to call ``self.run`` within a test request context.
         """
         with app.test_request_context():
             return self.run(*args, **kwargs)
@@ -151,34 +222,6 @@ class Command(object):
         """
         raise NotImplementedError
 
-    def prompt(self, name, default=None):
-        warnings.warn_explicit(
-            "Command.prompt is deprecated, use prompt() function instead")
-
-        prompt(name, default)
-
-    def prompt_pass(self, name, default=None):
-        warnings.warn_explicit(
-            "Command.prompt_pass is deprecated, use prompt_pass() function "
-            "instead")
-
-        prompt_pass(name, default)
-
-    def prompt_bool(self, name, default=False):
-        warnings.warn_explicit(
-            "Command.prompt_bool is deprecated, use prompt_bool() function "
-            "instead")
-
-        prompt_bool(name, default)
-
-    def prompt_choices(self, name, choices, default=None):
-        warnings.warn_explicit(
-            "Command.choices is deprecated, use prompt_choices() function "
-            "instead")
-
-        prompt_choices(name, choices, default)
-
-
 class Shell(Command):
     """
     Runs a Python shell inside Flask application context.
@@ -216,11 +259,13 @@ class Shell(Command):
             Option('--no-ipython',
                 action="store_true",
                 dest='no_ipython',
-                default=not(self.use_ipython)),
+                default=not(self.use_ipython),
+                help="Do not use the BPython shell"),
             Option('--no-bpython',
                 action="store_true",
                 dest='no_bpython',
-                default=not(self.use_bpython))
+                default=not(self.use_bpython),
+                help="Do not use the IPython shell"),
         )
 
     def get_context(self):
@@ -273,12 +318,15 @@ class Server(Command):
 
     :param host: server host
     :param port: server port
-    :param use_debugger: if False, will no longer use Werkzeug debugger.
+    :param use_debugger: Flag whether to default to using the Werkzeug debugger.
                          This can be overriden in the command line
-                         by passing the **-d** flag.
-    :param use_reloader: if False, will no longer use auto-reloader.
+                         by passing the **-d** or **-D** flag.
+                         Defaults to False, for security.
+
+    :param use_reloader: Flag whether to use the auto-reloader.
+                         Default to True when debugging.
                          This can be overriden in the command line by
-                         passing the **-r** flag.
+                         passing the **-r**/**-R** flag.
     :param threaded: should the process handle each request in a separate
                      thread?
     :param processes: number of processes to spawn
@@ -288,14 +336,14 @@ class Server(Command):
 
     help = description = 'Runs the Flask development server i.e. app.run()'
 
-    def __init__(self, host='127.0.0.1', port=5000, use_debugger=True,
-                 use_reloader=True, threaded=False, processes=1,
+    def __init__(self, host='127.0.0.1', port=5000, use_debugger=None,
+                 use_reloader=None, threaded=False, processes=1,
                  passthrough_errors=False, **options):
 
         self.port = port
         self.host = host
         self.use_debugger = use_debugger
-        self.use_reloader = use_reloader
+        self.use_reloader = use_reloader if use_reloader is not None else use_debugger
         self.server_options = options
         self.threaded = threaded
         self.processes = processes
@@ -304,7 +352,7 @@ class Server(Command):
     def get_options(self):
 
         options = (
-            Option('-t', '--host',
+            Option('-h', '--host',
                    dest='host',
                    default=self.host),
 
@@ -327,39 +375,45 @@ class Server(Command):
                    action='store_true',
                    dest='passthrough_errors',
                    default=self.passthrough_errors),
-        )
 
-        if self.use_debugger:
-            options += (Option('-d', '--no-debug',
-                               action='store_false',
-                               dest='use_debugger',
-                               default=self.use_debugger),)
+            Option('-d', '--debug',
+                   action='store_true',
+                   dest='use_debugger',
+                   help='enable the Werkzeug debugger (DO NOT use in production code)',
+                   default=self.use_debugger),
+            Option('-D', '--no-debug',
+                   action='store_false',
+                   dest='use_debugger',
+                   help='disable the Werkzeug debugger',
+                   default=self.use_debugger),
 
-        else:
-            options += (Option('-d', '--debug',
-                               action='store_true',
-                               dest='use_debugger',
-                               default=self.use_debugger),)
-
-        if self.use_reloader:
-            options += (Option('-r', '--no-reload',
-                               action='store_false',
-                               dest='use_reloader',
-                               default=self.use_reloader),)
-
-        else:
-            options += (Option('-r', '--reload',
-                               action='store_true',
-                               dest='use_reloader',
-                               default=self.use_reloader),)
+            Option('-r', '--reload',
+                   action='store_true',
+                   dest='use_reloader',
+                   help='monitor Python files for changes (not 100% safe for production use)',
+                   default=self.use_reloader),
+            Option('-R', '--no-reload',
+                   action='store_false',
+                   dest='use_reloader',
+                   help='do not monitor Python files for changes',
+                   default=self.use_reloader),
+            )
 
         return options
 
-    def handle(self, app, host, port, use_debugger, use_reloader,
+    def __call__(self, app, host, port, use_debugger, use_reloader,
                threaded, processes, passthrough_errors):
         # we don't need to run the server in request context
         # so just run it directly
 
+        if use_debugger is None:
+            use_debugger = app.debug
+            if use_debugger is None:
+                use_debugger = True
+                if sys.stderr.isatty():
+                    print("Debugging is on. DANGER: Do not allow random users to connect to this server.", file=sys.stderr)
+        if use_reloader is None:
+            use_reloader = app.debug
         app.run(host=host,
                 port=port,
                 debug=use_debugger,
@@ -453,3 +507,5 @@ class ShowUrls(Command):
 
         for row in rows:
             print(str_template % row[:column_length])
+
+
